@@ -169,9 +169,29 @@ static const char** get_signal_strength_icon(int signal_strength)
     return NULL;
 }
 
+
+
+static void ctk_3d_vision_pro_finalize(GObject *object)
+{
+    Ctk3DVisionPro *ctk_object = CTK_3D_VISION_PRO(object);
+
+    g_signal_handlers_disconnect_matched(G_OBJECT(ctk_object->ctk_event),
+                                         G_SIGNAL_MATCH_DATA,
+                                         0,
+                                         0,
+                                         NULL,
+                                         NULL,
+                                         (gpointer) ctk_object);
+}
+
+
+
 static void ctk_3d_vision_pro_class_init(Ctk3DVisionProClass
                                          *ctk_3d_vision_pro_class)
 {
+    GObjectClass *gobject_class = (GObjectClass *)ctk_3d_vision_pro_class;
+    gobject_class->finalize = ctk_3d_vision_pro_finalize;
+    
     __signals[CHANGED] = g_signal_new("changed",
                        G_OBJECT_CLASS_TYPE(ctk_3d_vision_pro_class),
                        G_SIGNAL_RUN_LAST,
@@ -180,6 +200,8 @@ static void ctk_3d_vision_pro_class_init(Ctk3DVisionProClass
                        G_TYPE_NONE, 0);
 }
 
+
+
 GType ctk_3d_vision_pro_get_type(void)
 {
     static GType ctk_3d_vision_pro_type = 0;
@@ -260,7 +282,7 @@ static GtkWidget *create_glasses_list_menu(Ctk3DVisionPro *ctk_3d_vision_pro,
     int i;
 
     mnu_glasses_name = (CtkDropDownMenu *)
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
     g_signal_connect(G_OBJECT(mnu_glasses_name), "changed",
                      G_CALLBACK(glasses_name_changed),
                      (gpointer) dlg);
@@ -674,7 +696,7 @@ static void enable_widgets(Ctk3DVisionPro *ctk_3d_vision_pro, Bool enable)
     gtk_widget_set_sensitive(ctk_3d_vision_pro->table.hscrollbar, enable);
 }
 
-static void svp_config_changed(GtkWidget *widget, gpointer arg1,
+static void svp_config_changed(GtkObject *object, gpointer arg1,
                                gpointer user_data)
 {
     CtkEventStruct *event_struct;
@@ -696,7 +718,7 @@ static void svp_config_changed(GtkWidget *widget, gpointer arg1,
     case NV_CTRL_3D_VISION_PRO_TRANSCEIVER_MODE:
         {
             SVP_RANGE range;
-            CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(widget);
+            CtkDropDownMenu *menu = CTK_DROP_DOWN_MENU(ctk_3d_vision_pro->menu);
             range = ctk_drop_down_menu_get_current_value(menu);
 
             if (range != CHANNEL_RANGE_TO_OPTION_MENU_IDX(event_struct->value)) {
@@ -1458,6 +1480,7 @@ GtkWidget* ctk_3d_vision_pro_new(NvCtrlAttributeHandle *handle,
 
     ctk_3d_vision_pro->handle = handle;
     ctk_3d_vision_pro->ctk_config = ctk_config;
+    ctk_3d_vision_pro->ctk_event = ctk_event;
     ctk_3d_vision_pro->add_glasses_dlg = NULL;
 
     // populate ctk_3d_vision_pro...
@@ -1664,7 +1687,7 @@ GtkWidget* ctk_3d_vision_pro_new(NvCtrlAttributeHandle *handle,
 
     hbox = gtk_hbox_new(FALSE, 5);
     menu = (CtkDropDownMenu *)
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
     ctk_drop_down_menu_append_item(menu, "Short Range (up to 5 meters)", 0);
     ctk_drop_down_menu_append_item(menu, "Medium Range (up to 15 meters)", 1);
     ctk_drop_down_menu_append_item(menu, "Long Range", 2);
@@ -1771,7 +1794,7 @@ GtkTextBuffer *ctk_3d_vision_pro_create_help(GtkTextTagTable *table)
                          "with the hub.");
 
     ctk_help_heading(b, &i, "Add glasses");
-    ctk_help_para(b, &i, __add_glasses_tooltip);
+    ctk_help_para(b, &i, "%s", __add_glasses_tooltip);
     ctk_help_para(b, &i, "This action is used to set up new 3D Vision Pro Glasses. "
                          "On clicking this button the hub enters into pairing mode. "
                          "Follow the instructions on Add Glasses dialog box. "
@@ -1780,7 +1803,7 @@ GtkTextBuffer *ctk_3d_vision_pro_create_help(GtkTextTagTable *table)
                          " glasses or 'Cancel' if do not wish to store them.");
 
     ctk_help_heading(b, &i, "Refresh Glasses' Information");
-    ctk_help_para(b, &i, __refresh_tooltip);
+    ctk_help_para(b, &i, "%s", __refresh_tooltip);
     ctk_help_para(b, &i, "Refresh glasses information is typically required when- \n"
                          "o Glasses move in and out of the range.\n"
                          "o Get the updated battery level of all the glasses.");
@@ -1792,18 +1815,18 @@ GtkTextBuffer *ctk_3d_vision_pro_create_help(GtkTextTagTable *table)
                          "for a few seconds.");
 
     ctk_help_heading(b, &i, "Rename glasses");
-    ctk_help_para(b, &i, __rename_tooltip);
+    ctk_help_para(b, &i, "%s", __rename_tooltip);
     ctk_help_para(b, &i, "Select the glasses from the list of paired glasses "
                          "that you want to rename and provide an unique new name.");
 
     ctk_help_heading(b, &i, "Remove glasses");
-    ctk_help_para(b, &i, __remove_glasses_tooltip);
+    ctk_help_para(b, &i, "%s", __remove_glasses_tooltip);
     ctk_help_para(b, &i, "Select the glasses from the list of paired glasses "
                          "that you want to remove. On removal glasses get "
                          "unpaired and will not sync to the hub.");
 
     ctk_help_heading(b, &i, "Glasses Information");
-    ctk_help_para(b, &i, __goggle_info_tooltip);
+    ctk_help_para(b, &i, "%s", __goggle_info_tooltip);
 
     ctk_help_heading(b, &i, "Glasses Name");
     ctk_help_para(b, &i, "Each pair of glasses has an unique name and the name should "
@@ -1832,7 +1855,7 @@ GtkTextBuffer *ctk_3d_vision_pro_create_help(GtkTextTagTable *table)
                          "\tNo Signal\t\t [0%%]");
 
     ctk_help_heading(b, &i, "Hub Range");
-    ctk_help_para(b, &i, __channel_range_tooltip);
+    ctk_help_para(b, &i, "%s", __channel_range_tooltip);
     ctk_help_para(b, &i, "The hub range is the farthest distance that the "
                          "glasses can synchronize with the 3D Vision Pro Hub. "
                          "You can reduce the hub range to limit the experience "
