@@ -4,22 +4,17 @@
  *
  * Copyright (C) 2009 NVIDIA Corporation.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of Version 2 of the GNU General Public
- * License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms and conditions of the GNU General Public License,
+ * version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See Version 2
- * of the GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
+ * more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the:
- *
- *           Free Software Foundation, Inc.
- *           59 Temple Place - Suite 330
- *           Boston, MA 02111-1307, USA
- *
+ * along with this program.  If not, see <http://www.gnu.org/licenses>.
  */
 
 #include <stdlib.h>
@@ -71,8 +66,9 @@ static const char *__reset_default_config_button_help =
 
 static void ecc_config_button_toggled(GtkWidget *, gpointer);
 static void show_ecc_toggle_warning_dlg(CtkEcc *);
+static void ecc_set_config_status(CtkEcc *);
 static void ecc_configuration_update_received(GtkObject *, gpointer, gpointer);
-static void post_ecc_configuration_update(CtkEcc *, gboolean);
+static void post_ecc_configuration_update(CtkEcc *);
 
 GType ctk_ecc_get_type(void)
 {
@@ -89,6 +85,7 @@ GType ctk_ecc_get_type(void)
             sizeof (CtkEcc),
             0,    /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_ecc_type =
@@ -195,18 +192,49 @@ static gboolean update_ecc_info(gpointer user_data)
  * post_ecc_configuration_update() - this function update status bar string.
  */
 
-static void post_ecc_configuration_update(CtkEcc *ctk_ecc, gboolean enabled)
+static void post_ecc_configuration_update(CtkEcc *ctk_ecc)
 {
-    ctk_config_statusbar_message(ctk_ecc->ctk_config,
-                                 "ECC %s.",
-                                 enabled ? "enabled" : "disabled");
+    gboolean configured = ctk_ecc->ecc_configured;
+    gboolean enabled = ctk_ecc->ecc_enabled;
+
+    const char *conf_string = configured ? "enabled" : "disabled";
+    char message[128];
+
+    if (configured != enabled) {
+        snprintf(message, sizeof(message), "ECC will be %s after reboot.",
+                 conf_string);
+    } else {
+        snprintf(message, sizeof(message), "ECC %s.", conf_string);
+    }
+
+    ctk_config_statusbar_message(ctk_ecc->ctk_config, "%s", message);
 } /* post_ecc_configuration_update() */
 
 
 
+/*
+ * ecc_set_config_status() - set ECC configuration button status: caller
+ * should make sure ctk_ecc->ecc_configured is set correctly before calling.
+ */
+
+static void ecc_set_config_status(CtkEcc *ctk_ecc)
+{
+    g_signal_handlers_block_by_func(G_OBJECT(ctk_ecc->configuration_status),
+                                    G_CALLBACK(ecc_config_button_toggled),
+                                    (gpointer) ctk_ecc);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctk_ecc->configuration_status),
+                                 ctk_ecc->ecc_configured);
+
+    g_signal_handlers_unblock_by_func(G_OBJECT(ctk_ecc->configuration_status),
+                                      G_CALLBACK(ecc_config_button_toggled),
+                                      (gpointer) ctk_ecc);
+}
+
+
+
 /*
  * ecc_configuration_update_received() - this function is called when the
- * NV_CTRL_GPU_ECC_CONFIGURATION atribute is changed by another
+ * NV_CTRL_GPU_ECC_CONFIGURATION attribute is changed by another
  * NV-CONTROL client.
  */
 
@@ -215,18 +243,14 @@ static void ecc_configuration_update_received(GtkObject *object,
 {
     CtkEventStruct *event_struct = (CtkEventStruct *) arg1;
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
-    
-    g_signal_handlers_block_by_func(G_OBJECT(ctk_ecc->configuration_status),
-                                    G_CALLBACK(ecc_config_button_toggled),
-                                    (gpointer) ctk_ecc);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctk_ecc->configuration_status),
-                                 event_struct->value);
 
-    g_signal_handlers_unblock_by_func(G_OBJECT(ctk_ecc->configuration_status),
-                                      G_CALLBACK(ecc_config_button_toggled),
-                                      (gpointer) ctk_ecc);
+    ctk_ecc->ecc_configured = event_struct->value;
+ 
+    /* set ECC configuration button status */
+    ecc_set_config_status(ctk_ecc);
+
     /* Update status bar message */
-    post_ecc_configuration_update(ctk_ecc, event_struct->value);
+    post_ecc_configuration_update(ctk_ecc);
 }
 
 
@@ -240,15 +264,31 @@ static void reset_default_config_button_clicked(GtkWidget *widget,
                                                 gpointer user_data)
 {
     gboolean status;
+    ReturnStatus ret;
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
     
     /* get default status and set it to ECC configuration */
-    NvCtrlGetAttribute(ctk_ecc->handle,
-                       NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION,
-                       &status);
-    NvCtrlSetAttribute(ctk_ecc->handle,
-                       NV_CTRL_GPU_ECC_CONFIGURATION,
-                       status);
+    ret = NvCtrlGetAttribute(ctk_ecc->handle,
+                             NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION,
+                             &status);
+    if (ret != NvCtrlSuccess) {
+        ctk_config_statusbar_message(ctk_ecc->ctk_config,
+                                     "Failed to get default configuration!");
+        return;
+    }
+    ret =  NvCtrlSetAttribute(ctk_ecc->handle,
+                              NV_CTRL_GPU_ECC_CONFIGURATION,
+                              status);
+    if (ret != NvCtrlSuccess) {
+        ctk_config_statusbar_message(ctk_ecc->ctk_config,
+                                     "Failed to set default configuration!");
+        return;
+    }
+
+    ctk_ecc->ecc_configured = status;
+
+    /* update ECC configuration button status */
+    ecc_set_config_status(ctk_ecc);
 
     /* show popup dialog*/
     show_ecc_toggle_warning_dlg(ctk_ecc);
@@ -335,20 +375,29 @@ static void ecc_config_button_toggled(GtkWidget *widget,
 {
     gboolean enabled;
     CtkEcc *ctk_ecc = CTK_ECC(user_data);
+    ReturnStatus ret;
+
     enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
     /* show popup dialog when user first time click ECC config */
     show_ecc_toggle_warning_dlg(ctk_ecc);
 
     /* set the newly specified ECC value */
-    NvCtrlSetAttribute(ctk_ecc->handle,
-                       NV_CTRL_GPU_ECC_CONFIGURATION,
-                       enabled);
+    ret = NvCtrlSetAttribute(ctk_ecc->handle,
+                             NV_CTRL_GPU_ECC_CONFIGURATION,
+                             enabled);
+    if (ret != NvCtrlSuccess) {
+        ctk_config_statusbar_message(ctk_ecc->ctk_config,
+                                     "Failed to set ECC configuration!");
+        return;
+    }
+    
+    ctk_ecc->ecc_configured = enabled;
 
     gtk_widget_set_sensitive(ctk_ecc->reset_default_config_button, TRUE);
 
     /* Update status bar message */
-    post_ecc_configuration_update(ctk_ecc, enabled);
+    post_ecc_configuration_update(ctk_ecc);
 } /* ecc_config_button_toggled() */
 
 
@@ -371,6 +420,7 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     gboolean ecc_default_status;
     ReturnStatus ret;
     gchar *ecc_enabled_string;
+    gchar *str = NULL;
 
     /* make sure we have a handle */
 
@@ -415,6 +465,16 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     }
     ctk_ecc->ecc_enabled = ecc_enabled; 
 
+    /* Query ECC Configuration */
+
+    ret = NvCtrlGetAttribute(handle, NV_CTRL_GPU_ECC_CONFIGURATION, &val);
+    if (ret != NvCtrlSuccess ||
+        val == NV_CTRL_GPU_ECC_CONFIGURATION_DISABLED) {
+            ctk_ecc->ecc_configured = FALSE;
+    } else {
+            ctk_ecc->ecc_configured = TRUE;
+    }
+
     /* get default status */
     ret = NvCtrlGetAttribute(handle,
                              NV_CTRL_GPU_ECC_DEFAULT_CONFIGURATION,
@@ -534,7 +594,7 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
     gtk_container_set_border_width(GTK_CONTAINER(hbox2), 5);
     gtk_box_pack_start(GTK_BOX(vbox), hbox2, FALSE, FALSE, 0);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctk_ecc->configuration_status),
-                                 ecc_enabled);
+                                 ctk_ecc->ecc_configured);
     ctk_config_set_tooltip(ctk_config, ctk_ecc->configuration_status,
                            __configuration_status_help);
     g_signal_connect(G_OBJECT(ctk_ecc->configuration_status), "clicked",
@@ -593,12 +653,16 @@ GtkWidget* ctk_ecc_new(NvCtrlAttributeHandle *handle,
                      (gpointer) ctk_ecc);
 
     /* Register a timer callback to update Ecc status info */
+    str = g_strdup_printf("ECC Settings (GPU %d)",
+                        NvCtrlGetTargetId(handle));
 
     ctk_config_add_timer(ctk_ecc->ctk_config,
                          DEFAULT_UPDATE_ECC_STATUS_INFO_TIME_INTERVAL,
-                         "ECC Settings",
+                         str,
                          (GSourceFunc) update_ecc_info,
                          (gpointer) ctk_ecc);
+
+    g_free(str);
     
     gtk_widget_show_all(GTK_WIDGET(ctk_ecc));
 
@@ -618,36 +682,36 @@ GtkTextBuffer *ctk_ecc_create_help(GtkTextTagTable *table,
     gtk_text_buffer_get_iter_at_offset(b, &i, 0);
 
     ctk_help_heading(b, &i, "ECC Settings Help");
-    ctk_help_para(b, &i, __ecc_settings_help);
+    ctk_help_para(b, &i, "%s", __ecc_settings_help);
     
     ctk_help_heading(b, &i, "ECC");
-    ctk_help_para(b, &i, __ecc_status_help);
+    ctk_help_para(b, &i, "%s", __ecc_status_help);
 
     if (ctk_ecc->dbit_error_available) {
         ctk_help_heading(b, &i, "Double-bit ECC Errors");
-        ctk_help_para(b, &i, __dbit_error_help);
+        ctk_help_para(b, &i, "%s", __dbit_error_help);
     }
     if (ctk_ecc->aggregate_dbit_error_available) {
         ctk_help_heading(b, &i, "Aggregate Double-bit ECC Errors");
-        ctk_help_para(b, &i, __aggregate_dbit_error_help);
+        ctk_help_para(b, &i, "%s", __aggregate_dbit_error_help);
     }
     ctk_help_heading(b, &i, "ECC Configuration");
-    ctk_help_para(b, &i, __configuration_status_help);
+    ctk_help_para(b, &i, "%s", __configuration_status_help);
 
     ctk_help_heading(b, &i, "Enable ECC");
-    ctk_help_para(b, &i, __ecc_status_help);
+    ctk_help_para(b, &i, "%s", __ecc_status_help);
 
     if (ctk_ecc->dbit_error_available) {
         ctk_help_heading(b, &i, "Clear ECC Errors");
-        ctk_help_para(b, &i, __clear_button_help);
+        ctk_help_para(b, &i, "%s", __clear_button_help);
     }
     if (ctk_ecc->aggregate_dbit_error_available) {
         ctk_help_heading(b, &i, "Clear Aggregate ECC Errors");
-        ctk_help_para(b, &i, __clear_aggregate_button_help);
+        ctk_help_para(b, &i, "%s", __clear_aggregate_button_help);
     }
     
     ctk_help_heading(b, &i, "Reset Default Configuration");
-    ctk_help_para(b, &i, __reset_default_config_button_help);
+    ctk_help_para(b, &i, "%s", __reset_default_config_button_help);
 
     ctk_help_finish(b);
 
