@@ -218,7 +218,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_MAX_SCREEN_WIDTH);
     MAKE_SIGNAL(NV_CTRL_MAX_SCREEN_HEIGHT);
     MAKE_SIGNAL(NV_CTRL_MAX_DISPLAYS);
-    MAKE_SIGNAL(NV_CTRL_DYNAMIC_TWINVIEW);
     MAKE_SIGNAL(NV_CTRL_MULTIGPU_DISPLAY_OWNER);
     MAKE_SIGNAL(NV_CTRL_GPU_SCALING);
     MAKE_SIGNAL(NV_CTRL_GPU_SCALING_DEFAULT_TARGET);
@@ -237,7 +236,6 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_GVO_OUTPUT_VIDEO_LOCKED);
     MAKE_SIGNAL(NV_CTRL_GVO_SYNC_LOCK_STATUS);
     MAKE_SIGNAL(NV_CTRL_GVO_ANC_TIME_CODE_GENERATION);
-    MAKE_SIGNAL(NV_CTRL_ONDEMAND_VBLANK_INTERRUPTS);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_ALPHA_KEY);
     MAKE_SIGNAL(NV_CTRL_GVO_COMPOSITE_NUM_KEY_RANGES);
@@ -329,6 +327,16 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_BASE_MOSAIC);
     MAKE_SIGNAL(NV_CTRL_MULTIGPU_MASTER_POSSIBLE);
     MAKE_SIGNAL(NV_CTRL_GPU_POWER_MIZER_DEFAULT_MODE);
+    MAKE_SIGNAL(NV_CTRL_XV_SYNC_TO_DISPLAY_ID);
+    MAKE_SIGNAL(NV_CTRL_BACKLIGHT_BRIGHTNESS);
+    MAKE_SIGNAL(NV_CTRL_GPU_LOGO_BRIGHTNESS);
+    MAKE_SIGNAL(NV_CTRL_GPU_SLI_LOGO_BRIGHTNESS);
+    MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_SPEED);
+    MAKE_SIGNAL(NV_CTRL_PALETTE_UPDATE_EVENT);
+    MAKE_SIGNAL(NV_CTRL_VIDEO_ENCODER_UTILIZATION);
+    MAKE_SIGNAL(NV_CTRL_GSYNC_ALLOWED);
+    MAKE_SIGNAL(NV_CTRL_GPU_NVCLOCK_OFFSET);
+    MAKE_SIGNAL(NV_CTRL_GPU_MEM_TRANSFER_RATE_OFFSET);
 #undef MAKE_SIGNAL
 
     /*
@@ -338,7 +346,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GPU_POWER_MIZER_DEFAULT_MODE
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GPU_MEM_TRANSFER_RATE_OFFSET
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -390,6 +398,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_TARGET_INDEX);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_DISPLAY_NAME_RANDR);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_UUID);
+    MAKE_STRING_SIGNAL(NV_CTRL_STRING_GPU_UTILIZATION);
     MAKE_STRING_SIGNAL(NV_CTRL_STRING_MULTIGPU_MODE);
 #undef MAKE_STRING_SIGNAL
 
@@ -425,9 +434,10 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_ENABLED_ON_XSCREEN);
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_ASSIGNED_TO_XSCREEN);
     MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_GPU_FLAGS);
+    MAKE_BINARY_SIGNAL(NV_CTRL_BINARY_DATA_DISPLAYS_ON_GPU);
 #undef MAKE_BINARY_SIGNAL
 
-#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_GPU_FLAGS
+#if NV_CTRL_BINARY_DATA_LAST_ATTRIBUTE != NV_CTRL_BINARY_DATA_DISPLAYS_ON_GPU
 #warning "There are attributes that do not emit signals!"
 #endif
 
@@ -724,6 +734,7 @@ static gboolean ctk_event_dispatch(GSource *source,
             event_struct.value        = nvctrlevent->value;
             event_struct.display_mask = nvctrlevent->display_mask;
             event_struct.is_availability_changed = TRUE;
+            event_struct.availability = nvctrlevent->availability;
 
             /*
              * XXX Is emitting a signal with g_signal_emit() really
