@@ -704,7 +704,7 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
     /* Create the menu */
 
     d = (CtkDropDownMenu *)
-        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_COMBO);
+        ctk_drop_down_menu_new(CTK_DROP_DOWN_MENU_FLAG_READONLY);
     
     for (i = 0; i < ARRAY_LEN(applicationSettings); i++) {
         ctk_drop_down_menu_append_item(d, applicationSettings[i], i);
@@ -723,8 +723,8 @@ static GtkWidget *create_fsaa_setting_menu(CtkMultisample *ctk_multisample,
     /* set the menu item */
     ctk_drop_down_menu_set_current_value(d, idx);
 
-    ctk_config_set_tooltip(ctk_multisample->ctk_config, d->menu,
-                           __aa_menu_help);
+    ctk_drop_down_menu_set_tooltip(ctk_multisample->ctk_config, d,
+                                   __aa_menu_help);
 
     g_signal_connect(G_OBJECT(d),
                      "changed",
@@ -848,12 +848,10 @@ static void fsaa_setting_menu_changed(GtkObject *object, gpointer user_data)
     gint idx;
     gboolean override;
     gboolean enhance;
-    
-    CTK_DROP_DOWN_MENU(ctk_multisample->fsaa_menu)->current_selected_item_widget
-        = GTK_WIDGET(object);
+
     idx = ctk_drop_down_menu_get_current_value
-        (CTK_DROP_DOWN_MENU(ctk_multisample->fsaa_menu)); 
-    
+        (CTK_DROP_DOWN_MENU(ctk_multisample->fsaa_menu));
+
     /* The FSAA dropdown menu is setup this way:
      *
      * 0 == app
@@ -1005,7 +1003,8 @@ static void update_fxaa_from_fsaa_change(CtkMultisample *ctk_multisample,
     gboolean fxaa_value;
 
     /* The FSAA dropdown menu is: 0 == app, 1 == override, 2 == enhance */
-    gint fsaa_idx = CTK_DROP_DOWN_MENU(fsaa_menu)->current_selected_item;
+    gint fsaa_idx =
+        ctk_drop_down_menu_get_current_value(CTK_DROP_DOWN_MENU(fsaa_menu));
     
     if (fsaa_value != NV_CTRL_FSAA_MODE_NONE) {
         g_signal_handlers_block_by_func(G_OBJECT(fxaa_checkbox),
@@ -1043,7 +1042,8 @@ static void update_fsaa_from_fxaa_change (CtkMultisample *ctk_multisample,
     GtkWidget *fsaa_menu = ctk_multisample->fsaa_menu;
 
     /* The FSAA dropdown menu is: 0 == app, 1 == override, 2 == enhance */
-    gint fsaa_idx = CTK_DROP_DOWN_MENU(fsaa_menu)->current_selected_item;
+    gint fsaa_idx =
+        ctk_drop_down_menu_get_current_value(CTK_DROP_DOWN_MENU(fsaa_menu));
     
     gint fsaa_val;
 
