@@ -34,6 +34,8 @@ static void post_vblank_sync_button_toggled(CtkOpenGL *, gboolean);
 
 static void post_allow_flipping_button_toggled(CtkOpenGL *, gboolean);
 
+static void post_allow_gsync_button_toggled(CtkOpenGL *, gboolean);
+
 static void post_force_stereo_button_toggled(CtkOpenGL *, gboolean);
 
 static void post_show_sli_visual_indicator_button_toggled(CtkOpenGL *,
@@ -48,8 +50,12 @@ static void post_stereo_eyes_exchange_button_toggled(CtkOpenGL *, gboolean);
 
 static void post_aa_line_gamma_toggled(CtkOpenGL *, gboolean);
 
+static void post_use_conformant_clamping_button_toggled(CtkOpenGL *, gint);
+
 static void allow_flipping_button_toggled(GtkWidget *, gpointer);
 
+static void allow_gsync_button_toggled(GtkWidget *, gpointer);
+
 static void force_stereo_button_toggled (GtkWidget *, gpointer);
 
 static void xinerama_stereo_button_toggled (GtkWidget *, gpointer);
@@ -58,6 +64,8 @@ static void stereo_eyes_exchange_button_toggled (GtkWidget *, gpointer);
 
 static void aa_line_gamma_toggled        (GtkWidget *, gpointer);
 
+static void use_conformant_clamping_button_toggled(GtkWidget *, gpointer);
+
 static void show_sli_visual_indicator_button_toggled (GtkWidget *, gpointer);
 
 static void show_multigpu_visual_indicator_button_toggled (GtkWidget *, gpointer);
@@ -142,6 +150,12 @@ static const char *__stereo_eyes_exchange_help =
 "eye image in the right eye and vice versa for stereo "
 "drawables.  This option is applied immediately.";
 
+static const char *__use_conformant_clamping_help =
+"Disabling this option causes OpenGL to replace GL_CLAMP with "
+"GL_CLAMP_TO_EDGE for borderless 2D textures.  This eliminates "
+"seams at the edges of textures in some older games such as "
+"Quake 3.";
+
 #define __SYNC_TO_VBLANK      (1 << 1)
 #define __ALLOW_FLIPPING      (1 << 2)
 #define __AA_LINE_GAMMA_VALUE (1 << 3)
@@ -153,6 +167,8 @@ static const char *__stereo_eyes_exchange_help =
 #define __SHOW_SLI_VISUAL_INDICATOR        (1 << 9)
 #define __STEREO_EYES_EXCHANGE (1 << 10)
 #define __SHOW_MULTIGPU_VISUAL_INDICATOR    (1 << 11)
+#define __CONFORMANT_CLAMPING (1 << 12)
+#define __ALLOW_GSYNC         (1 << 13)
 
 
 
@@ -173,6 +189,7 @@ GType ctk_opengl_get_type(
             sizeof (CtkOpenGL),
             0, /* n_preallocs */
             NULL, /* instance_init */
+            NULL  /* value_table */
         };
 
         ctk_opengl_type = g_type_register_static (GTK_TYPE_VBOX,
@@ -200,22 +217,26 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
 
     gint sync_to_vblank;
     gint flipping_allowed;
+    gint gsync_allowed;
     gint force_stereo;
     gint xinerama_stereo;
     gint stereo_eyes_exchange;
     NVCTRLAttributeValidValuesRec image_settings_valid;
     gint image_settings_value;
     gint aa_line_gamma;
+    gint use_conformant_clamping;
     gint show_sli_visual_indicator;
     gint show_multigpu_visual_indicator;
 
     ReturnStatus ret_sync_to_vblank;
     ReturnStatus ret_flipping_allowed;
+    ReturnStatus ret_gsync_allowed;
     ReturnStatus ret_force_stereo;
     ReturnStatus ret_xinerama_stereo;
     ReturnStatus ret_stereo_eyes_exchange;
     ReturnStatus ret_image_settings;
     ReturnStatus ret_aa_line_gamma;
+    ReturnStatus ret_use_conformant_clamping;
     ReturnStatus ret_show_sli_visual_indicator;
     ReturnStatus ret_show_multigpu_visual_indicator;
 
@@ -228,6 +249,10 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         NvCtrlGetAttribute(handle, NV_CTRL_FLIPPING_ALLOWED,
                            &flipping_allowed);
 
+    ret_gsync_allowed =
+        NvCtrlGetAttribute(handle, NV_CTRL_GSYNC_ALLOWED,
+                           &gsync_allowed);
+
     ret_force_stereo =
         NvCtrlGetAttribute(handle, NV_CTRL_FORCE_STEREO, &force_stereo);
 
@@ -252,6 +277,11 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
 
     ret_aa_line_gamma = NvCtrlGetAttribute(handle, NV_CTRL_OPENGL_AA_LINE_GAMMA,
                                            &aa_line_gamma);
+
+    ret_use_conformant_clamping =
+        NvCtrlGetAttribute(handle, NV_CTRL_TEXTURE_CLAMPING,
+                           &use_conformant_clamping);
+
     ret_show_sli_visual_indicator = NvCtrlGetAttribute(handle,
                                           NV_CTRL_SHOW_SLI_VISUAL_INDICATOR,
                                           &show_sli_visual_indicator);
@@ -262,11 +292,13 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
     /* There are no OpenGL settings to change (OpenGL disabled?) */
     if ((ret_sync_to_vblank != NvCtrlSuccess) &&
         (ret_flipping_allowed != NvCtrlSuccess) &&
+        (ret_gsync_allowed != NvCtrlSuccess) &&
         (ret_force_stereo != NvCtrlSuccess) &&
         (ret_xinerama_stereo != NvCtrlSuccess) &&
         (ret_stereo_eyes_exchange != NvCtrlSuccess) &&
         (ret_image_settings != NvCtrlSuccess) &&
         (ret_aa_line_gamma != NvCtrlSuccess) &&
+        (ret_use_conformant_clamping != NvCtrlSuccess) &&
         (ret_show_sli_visual_indicator != NvCtrlSuccess) &&
         (ret_show_multigpu_visual_indicator != NvCtrlSuccess)) {
         return NULL;
@@ -374,7 +406,41 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
         
         ctk_opengl->allow_flipping_button = check_button;
     }
-    
+
+    /*
+     * allow G-SYNC
+     *
+     * Always create the checkbox, but only show it if the attribute starts out
+     * available.
+     */
+
+    label = gtk_label_new("Allow G-SYNC");
+
+    check_button = gtk_check_button_new();
+    gtk_container_add(GTK_CONTAINER(check_button), label);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
+                                 gsync_allowed);
+
+    gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
+
+    g_signal_connect(G_OBJECT(check_button), "toggled",
+                     G_CALLBACK(allow_gsync_button_toggled),
+                     (gpointer) ctk_opengl);
+
+    g_signal_connect(G_OBJECT(ctk_event),
+                     CTK_EVENT_NAME(NV_CTRL_GSYNC_ALLOWED),
+                     G_CALLBACK(value_changed), (gpointer) ctk_opengl);
+
+    ctk_config_set_tooltip(ctk_config, check_button,
+                           "Enabling this option allows OpenGL to flip "
+                           "using G-SYNC when possible.  This option is "
+                           "applied immediately.");
+
+    ctk_opengl->active_attributes |= __ALLOW_GSYNC;
+
+    ctk_opengl->allow_gsync_button = check_button;
+
     if (ret_force_stereo == NvCtrlSuccess) {
 
         label = gtk_label_new("Force Stereo Flipping");
@@ -565,6 +631,37 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
                                      gtk_toggle_button_get_active
                                      (GTK_TOGGLE_BUTTON(check_button)));
     }
+
+    /*
+     * NV_CTRL_TEXTURE_CLAMPING
+     */
+
+    if (ret_use_conformant_clamping == NvCtrlSuccess) {
+        label = gtk_label_new("Use Conformant Texture Clamping");
+
+        check_button = gtk_check_button_new();
+        gtk_container_add(GTK_CONTAINER(check_button), label);
+
+        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button),
+                                     use_conformant_clamping);
+
+        gtk_box_pack_start(GTK_BOX(vbox), check_button, FALSE, FALSE, 0);
+
+        g_signal_connect(G_OBJECT(check_button), "toggled",
+                         G_CALLBACK(use_conformant_clamping_button_toggled),
+                         (gpointer) ctk_opengl);
+
+        g_signal_connect(G_OBJECT(ctk_event),
+                         CTK_EVENT_NAME(NV_CTRL_TEXTURE_CLAMPING),
+                         G_CALLBACK(value_changed), (gpointer) ctk_opengl);
+
+        ctk_config_set_tooltip(ctk_config, check_button,
+                               __use_conformant_clamping_help);
+
+        ctk_opengl->active_attributes |= __CONFORMANT_CLAMPING;
+
+        ctk_opengl->use_conformant_clamping_button = check_button;
+    }
     
     if (ret_show_sli_visual_indicator == NvCtrlSuccess) {
 
@@ -624,6 +721,14 @@ GtkWidget* ctk_opengl_new(NvCtrlAttributeHandle *handle,
 
     gtk_widget_show_all(GTK_WIDGET(object));
 
+    /*
+     * If GSYNC is not currently available, start out with the GSYNC button
+     * hidden.
+     */
+    if (ret_gsync_allowed != NvCtrlSuccess) {
+        gtk_widget_hide(GTK_WIDGET(ctk_opengl->allow_gsync_button));
+    }
+
     return GTK_WIDGET(object);
 }
 
@@ -646,6 +751,14 @@ static void post_allow_flipping_button_toggled(CtkOpenGL *ctk_opengl,
                                  enabled ? "allowed" : "not allowed");
 }
 
+static void post_allow_gsync_button_toggled(CtkOpenGL *ctk_opengl,
+                                          gboolean enabled)
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "G-SYNC %s.",
+                                 enabled ? "allowed" : "not allowed");
+}
+
 static void post_force_stereo_button_toggled(CtkOpenGL *ctk_opengl, 
                                              gboolean enabled)
 {
@@ -696,6 +809,15 @@ static void post_aa_line_gamma_toggled(CtkOpenGL *ctk_opengl,
                                  enabled ? "enabled" : "disabled");
 }
 
+static void
+post_use_conformant_clamping_button_toggled(CtkOpenGL *ctk_opengl,
+                                               int clamping) 
+{
+    ctk_config_statusbar_message(ctk_opengl->ctk_config,
+                                 "Use %sConformant OpenGL Texture Clamping",
+                                 (clamping == NV_CTRL_TEXTURE_CLAMPING_SPEC) ?
+                                 "" : "Non-");
+}
 
 static void vblank_sync_button_toggled(
     GtkWidget *widget,
@@ -730,6 +852,20 @@ static void allow_flipping_button_toggled(GtkWidget *widget,
     
 }
 
+static void allow_gsync_button_toggled(GtkWidget *widget,
+                                     gpointer user_data)
+{
+    CtkOpenGL *ctk_opengl;
+    gboolean enabled;
+
+    ctk_opengl = CTK_OPENGL(user_data);
+
+    enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
+    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_GSYNC_ALLOWED, enabled);
+    post_allow_gsync_button_toggled(ctk_opengl, enabled);
+}
+
 static void force_stereo_button_toggled(GtkWidget *widget,
                                          gpointer user_data)
 {
@@ -826,6 +962,24 @@ static void aa_line_gamma_toggled(
     post_aa_line_gamma_toggled(ctk_opengl, enabled);
 }
 
+static void use_conformant_clamping_button_toggled(GtkWidget *widget,
+                                                   gpointer user_data)
+{
+    CtkOpenGL *ctk_opengl;
+    int clamping;
+    
+    ctk_opengl = CTK_OPENGL(user_data);
+
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
+        clamping = NV_CTRL_TEXTURE_CLAMPING_SPEC;
+    } else {
+        clamping = NV_CTRL_TEXTURE_CLAMPING_EDGE;
+    }
+   
+    NvCtrlSetAttribute(ctk_opengl->handle, NV_CTRL_TEXTURE_CLAMPING, clamping);
+
+    post_use_conformant_clamping_button_toggled(ctk_opengl, clamping);
+}
 
 
 /*
@@ -837,6 +991,7 @@ static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
     CtkEventStruct *event_struct;
     CtkOpenGL *ctk_opengl;
     gboolean enabled;
+    gboolean check_available = FALSE;
     GtkToggleButton *button;
     GCallback func;
 
@@ -854,6 +1009,12 @@ static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
         func = G_CALLBACK(allow_flipping_button_toggled);
         post_allow_flipping_button_toggled(ctk_opengl, event_struct->value);
         break;
+    case NV_CTRL_GSYNC_ALLOWED:
+        button = GTK_TOGGLE_BUTTON(ctk_opengl->allow_gsync_button);
+        func = G_CALLBACK(allow_gsync_button_toggled);
+        post_allow_gsync_button_toggled(ctk_opengl, event_struct->value);
+        check_available = TRUE;
+        break;
     case NV_CTRL_FORCE_STEREO:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->force_stereo_button);
         func = G_CALLBACK(force_stereo_button_toggled);
@@ -877,6 +1038,13 @@ static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
         gtk_widget_set_sensitive(ctk_opengl->aa_line_gamma_scale,
                                  event_struct->value);
         break;
+    case NV_CTRL_TEXTURE_CLAMPING:
+        button =
+            GTK_TOGGLE_BUTTON(ctk_opengl->use_conformant_clamping_button);
+        func = G_CALLBACK(use_conformant_clamping_button_toggled);
+        post_use_conformant_clamping_button_toggled(ctk_opengl,
+                                                    event_struct->value);
+        break;
     case NV_CTRL_SHOW_SLI_VISUAL_INDICATOR:
         button = GTK_TOGGLE_BUTTON(ctk_opengl->show_sli_visual_indicator_button);
         func = G_CALLBACK(show_sli_visual_indicator_button_toggled);
@@ -902,7 +1070,15 @@ static void value_changed(GtkObject *object, gpointer arg1, gpointer user_data)
         gtk_toggle_button_set_active(button, event_struct->value);
         g_signal_handlers_unblock_by_func(button, func, ctk_opengl);
     }
-    
+
+    if (check_available && event_struct->is_availability_changed) {
+        if (event_struct->availability) {
+            gtk_widget_show(GTK_WIDGET(button));
+        } else {
+            gtk_widget_hide(GTK_WIDGET(button));
+        }
+    }
+
 } /* value_changed() */
 
 
@@ -1025,7 +1201,7 @@ static void slider_changed(GtkAdjustment *adjustment, gpointer user_data)
 
 /*
  * aa_line_gamma_update_received() - this function is called when the
- * NV_CTRL_OPENGL_AA_LINE_GAMMA_VALUE atribute is changed by another NV-CONTROL
+ * NV_CTRL_OPENGL_AA_LINE_GAMMA_VALUE attribute is changed by another NV-CONTROL
  * client.
  */
 
@@ -1121,7 +1297,7 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
 
     if (ctk_opengl->active_attributes & __SYNC_TO_VBLANK) {
         ctk_help_heading(b, &i, "Sync to VBlank");
-        ctk_help_para(b, &i, __sync_to_vblank_help);
+        ctk_help_para(b, &i, "%s", __sync_to_vblank_help);
     }
 
     if (ctk_opengl->active_attributes & __ALLOW_FLIPPING) {
@@ -1141,19 +1317,38 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
                       "after the option is set.");
     }
 
+    if (ctk_opengl->active_attributes & __ALLOW_GSYNC) {
+        ctk_help_heading(b, &i, "Allow G-SYNC");
+        ctk_help_para(b, &i, "Enabling this option allows OpenGL to use G-SYNC "
+                      "when available.  G-SYNC is a technology that allows a "
+                      "monitor to delay updating the screen until the GPU is "
+                      "ready to display a new frame.  Without G-SYNC, the GPU "
+                      "waits for the display to be ready to accept a new frame "
+                      "instead.");
+
+        ctk_help_para(b, &i, "Note that this option is applied immediately, "
+                      "unlike most other OpenGL options which are only "
+                      "applied to OpenGL applications that are started "
+                      "after the option is set.");
+
+        ctk_help_para(b, &i, "This option can be overridden on a "
+                      "per-application basis using the GLGSYNCAllowed "
+                      "application profile key.");
+    }
+
     if (ctk_opengl->active_attributes & __FORCE_STEREO) {
         ctk_help_heading(b, &i, "Force Stereo Flipping");
-        ctk_help_para(b, &i, __force_stereo_help);
+        ctk_help_para(b, &i, "%s", __force_stereo_help);
     }
     
     if (ctk_opengl->active_attributes & __XINERAMA_STEREO) {
         ctk_help_heading(b, &i, "Allow Xinerama Stereo Flipping");
-        ctk_help_para(b, &i, __xinerama_stereo_help);
+        ctk_help_para(b, &i, "%s", __xinerama_stereo_help);
     }
     
     if (ctk_opengl->active_attributes & __STEREO_EYES_EXCHANGE) {
         ctk_help_heading(b, &i, "Exchange Stereo Eyes");
-        ctk_help_para(b, &i, __stereo_eyes_exchange_help);
+        ctk_help_para(b, &i, "%s", __stereo_eyes_exchange_help);
     }
     
     if (ctk_opengl->active_attributes & __IMAGE_SETTINGS) {
@@ -1192,13 +1387,18 @@ GtkTextBuffer *ctk_opengl_create_help(GtkTextTagTable *table,
     if (ctk_opengl->active_attributes & __AA_LINE_GAMMA) {
         ctk_help_heading(b, &i, "Enable gamma correction for "
                          "antialiased lines");
-        ctk_help_para(b, &i, __aa_line_gamma_checkbox_help );
+        ctk_help_para(b, &i, "%s", __aa_line_gamma_checkbox_help );
     }
 
     if (ctk_opengl->active_attributes & __AA_LINE_GAMMA_VALUE) {
         ctk_help_heading(b, &i, "Set gamma correction for "
                          "antialiased lines");
-        ctk_help_para(b, &i, __aa_line_gamma_slider_help);
+        ctk_help_para(b, &i, "%s", __aa_line_gamma_slider_help);
+    }
+
+    if (ctk_opengl->active_attributes & __CONFORMANT_CLAMPING) {
+        ctk_help_heading(b, &i, "Use Conformant Texture Clamping");
+        ctk_help_para(b, &i, "%s", __use_conformant_clamping_help);
     }
 
     if (ctk_opengl->active_attributes & __SHOW_SLI_VISUAL_INDICATOR) {
