@@ -43,7 +43,6 @@ G_BEGIN_DECLS
 #define CTK_POWERMIZER_GET_CLASS(obj) \
     (G_TYPE_INSTANCE_GET_CLASS ((obj), CTK_TYPE_POWERMIZER, CtkPowermizerClass))
 
-
 typedef struct _CtkPowermizer       CtkPowermizer;
 typedef struct _CtkPowermizerClass  CtkPowermizerClass;
 
@@ -56,21 +55,30 @@ struct _CtkPowermizer
 
     GtkWidget *adaptive_clock_status;
     GtkWidget *gpu_clock;
-    GtkWidget *memory_clock;
+    GtkWidget *memory_transfer_rate;
     GtkWidget *processor_clock;
     GtkWidget *power_source;
     GtkWidget *performance_level;
     GtkWidget *performance_table_hbox;
+    GtkWidget *performance_table_hbox1;
     GtkWidget *powermizer_menu;
+    GtkWidget *powermizer_txt;
     GtkWidget *box_powermizer_menu;
 
+    gchar     *powermizer_menu_help;
+
     GtkWidget *configuration_button;
     gboolean  dp_enabled;
     gboolean  dp_toggle_warning_dlg_shown;
+    gboolean  hasDecoupledClock;
+    gboolean  hasEditablePerfLevel;
     gint      attribute;
-    gchar     *dp_enabled_string;
+    gint      powermizer_default_mode;
     GtkWidget *status;
 
+    GtkWidget *editable_perf_level_table;
+    gint      num_perf_levels;
+
     GtkWidget *link_width;
     GtkWidget *link_speed;
     gboolean  pcie_gen_queriable;
