@@ -53,7 +53,6 @@
 #include "ctkpowermizer.h"
 #include "ctkclocks.h"
 #include "ctkvcs.h"
-#include "ctkpowersavings.h"
 #include "ctk3dvisionpro.h"
 
 #include "ctkdisplaydevice.h"
@@ -341,47 +340,27 @@ static void tree_selection_changed(GtkTreeSelection *selection,
 
 
 
-/*
- * tree_view_key_event() - callback for additional keyboard events we
- * want to track (space and Return) to expand and collapse collapsible
- * categories in the treeview.
+/* 
+ * row_activated_event() - callback for row-activated event
+ * - handles key presses automatically
+ * - allows the mouse to collapse/expand the menu even when the
+ *   expand button/triangle is not working.
  */
 
-static gboolean tree_view_key_event(GtkWidget *tree_view, GdkEvent *event, 
-                                    gpointer user_data)
+static void row_activated_event(GtkTreeView        *view,
+                                GtkTreePath        *path,
+                                GtkTreeViewColumn  *col,
+                                gpointer            user_data)
 {
-    GdkEventKey *key_event = (GdkEventKey *) event;
     CtkWindow *ctk_window = CTK_WINDOW(user_data);
-
-    if ((key_event->keyval == GDK_space) ||
-        (key_event->keyval == GDK_Return)) {
-        
-        GtkTreeSelection* selection;
-        GtkTreeModel *model;
-        GtkTreeIter iter;
-        GtkTreePath* path;
-
-        selection = gtk_tree_view_get_selection(ctk_window->treeview);
-        
-        if (!gtk_tree_selection_get_selected(selection, &model, &iter))
-            return FALSE;
-        
-        path = gtk_tree_model_get_path(model, &iter);
-
-        if (gtk_tree_view_row_expanded(ctk_window->treeview, path)) {
-            gtk_tree_view_collapse_row(ctk_window->treeview, path);
-        } else {
-            gtk_tree_view_expand_row(ctk_window->treeview, path, FALSE);
-        }
-
-        gtk_tree_path_free(path);
-
-        return TRUE;
+    
+    if (gtk_tree_view_row_expanded(ctk_window->treeview, path)) {
+        gtk_tree_view_collapse_row(ctk_window->treeview, path);
+    } else {
+        gtk_tree_view_expand_row(ctk_window->treeview, path, FALSE);
     }
-    
-    return FALSE;
-    
-} /* tree_view_key_event() */
+}
+
 
 
 static gboolean has_randr_gamma(NvCtrlAttributeHandle *handle)
@@ -425,11 +404,14 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     GtkTextBuffer *help;
         
+    NvCtrlAttributeHandle *server_handle = NULL;
+
     CtkEvent *ctk_event;
     CtkConfig *ctk_config;
     
     gint column_offset, i;
     gboolean slimm_page_added; /* XXX Kludge to only show one SLIMM page */
+    gchar *driver_version;
     
     /* create the new object */
 
@@ -544,10 +526,10 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     ctk_window->treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
     g_object_unref(ctk_window->tree_store);
 
-    /* catch keyboard events to the tree view */
+    /* Added row activated event to the tree view */
 
-    g_signal_connect(ctk_window->treeview, "key_press_event",
-                     G_CALLBACK(tree_view_key_event), GTK_OBJECT(ctk_window));
+    g_signal_connect(ctk_window->treeview, "row_activated",
+                     G_CALLBACK(row_activated_event), GTK_OBJECT(ctk_window));
 
     selection = gtk_tree_view_get_selection(ctk_window->treeview);
 
@@ -584,27 +566,26 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
     if (h->targets[X_SCREEN_TARGET].n) {
 
-        NvCtrlAttributeHandle *screen_handle = NULL;
         GtkWidget *child;
         int i;
 
         /*
          * XXX For now, just use the first handle in the list
          *     to communicate with the X server for these two
-         *     pages.
+         *     pages and the app profile page below.
          */
 
         for (i = 0 ; i < h->targets[X_SCREEN_TARGET].n; i++) {
             if (h->targets[X_SCREEN_TARGET].t[i].h) {
-                screen_handle = h->targets[X_SCREEN_TARGET].t[i].h;
+                server_handle = h->targets[X_SCREEN_TARGET].t[i].h;
                 break;
             }
         }
-        if (screen_handle) {
+        if (server_handle) {
 
             /* X Server information */
             
-            child = ctk_server_new(screen_handle, ctk_config);
+            child = ctk_server_new(server_handle, ctk_config);
             add_page(child,
                      ctk_server_create_help(tag_table,
                                             CTK_SERVER(child)),
@@ -613,7 +594,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
 
             /* X Server Display Configuration */
 
-            child = ctk_display_config_new(screen_handle, ctk_config);
+            child = ctk_display_config_new(server_handle, ctk_config);
             if (child) {
                 add_page(child,
                          ctk_display_config_create_help(tag_table,
@@ -656,12 +637,14 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         screen_name = NvCtrlGetDisplayName(screen_handle);
 
         child = ctk_screen_new(screen_handle, ctk_event);
-        gtk_object_ref(GTK_OBJECT(child));
+        g_object_ref(GTK_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
-                           CTK_WINDOW_HELP_COLUMN, 
-                           ctk_screen_create_help(tag_table, screen_name), -1);
+                           CTK_WINDOW_HELP_COLUMN,
+                           ctk_screen_create_help(tag_table, CTK_SCREEN(child),
+                                                  screen_name),
+                           -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_CONFIG_FILE_ATTRIBUTES_FUNC_COLUMN,
                            NULL, -1);
@@ -810,7 +793,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         child = ctk_gpu_new(gpu_handle, h->targets[X_SCREEN_TARGET].t, ctk_event,
                             ctk_config);
 
-        gtk_object_ref(GTK_OBJECT(child));
+        g_object_ref(GTK_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -826,15 +809,6 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
                            CTK_WINDOW_UNSELECT_WIDGET_FUNC_COLUMN,
                            ctk_gpu_page_unselect, -1);
 
-        /* power savings */
-
-        child = ctk_power_savings_new(gpu_handle, ctk_config, ctk_event);
-        if (child) {
-            help = ctk_power_savings_create_help(tag_table, CTK_POWER_SAVINGS(child));
-            add_page(child, help, ctk_window, &iter, NULL,
-                     "Power Savings Settings", NULL, NULL, NULL);
-        }
-
         /* thermal information */
 
         child = ctk_thermal_new(gpu_handle, ctk_config, ctk_event);
@@ -928,7 +902,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_LABEL_COLUMN, vcs_name, -1);
         child = ctk_vcs_new(vcs_handle, ctk_config);
-        gtk_object_ref(GTK_OBJECT(child));
+        g_object_ref(GTK_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -977,7 +951,7 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_LABEL_COLUMN, gvi_name, -1);
         child = ctk_gvi_new(gvi_handle, ctk_config, ctk_event);
-        gtk_object_ref(GTK_OBJECT(child));
+        g_object_ref(GTK_OBJECT(child));
         gtk_tree_store_set(ctk_window->tree_store, &iter,
                            CTK_WINDOW_WIDGET_COLUMN, child, -1);
         gtk_tree_store_set(ctk_window->tree_store, &iter,
@@ -1040,7 +1014,23 @@ GtkWidget *ctk_window_new(ParsedAttribute *p, ConfigProperties *conf,
     }
 
     /* app profile configuration */
-    widget = ctk_app_profile_new(ctk_config);
+    if (server_handle)
+    {
+        ReturnStatus ret = NvCtrlGetStringAttribute(server_handle,
+                               NV_CTRL_STRING_NVIDIA_DRIVER_VERSION,
+                               &driver_version);
+        if (ret != NvCtrlSuccess) {
+            driver_version = NULL;
+        }
+    } else {
+        driver_version = NULL;
+    }
+
+    widget = ctk_app_profile_new(ctk_config, driver_version);
+
+    if (driver_version) {
+        XFree(driver_version);
+    }
 
     add_page(widget, ctk_app_profile_create_help(CTK_APP_PROFILE(widget), tag_table),
              ctk_window, NULL, NULL, "Application Profiles",
@@ -1438,16 +1428,18 @@ static void add_display_devices(CtkWindow *ctk_window, GtkTreeIter *iter,
          * saved to the RC file when the UI is closed.
          */
 
-        if (handles) {
-            display_handle = nv_get_target_handle(handles,
-                                                  NV_CTRL_TARGET_TYPE_DISPLAY,
-                                                  display_id);
-        } else {
-            display_handle = NULL;
-        }
+        display_handle = nv_get_target_handle(handles,
+                                              NV_CTRL_TARGET_TYPE_DISPLAY,
+                                              display_id);
 
         if (!display_handle) {
-            continue;
+            display_handle =
+                nv_add_target(handles, NvCtrlGetDisplayPtr(gpu_handle),
+                              NV_CTRL_TARGET_TYPE_DISPLAY, display_id);
+
+            if (!display_handle) {
+                continue;
+            }
         }
 
         /*
